//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// TETRunAction.cc
// \file   MRCP_GEANT4/External/src/TETRunAction.cc
// \author Haegin Han
//

#include "TETRunAction.hh"

TETRunAction::TETRunAction(TETModelImport* _tetData, G4String _output)
:tetData(_tetData), fRun(0), numOfEvent(0), runID(0), outputFile(_output)
{}

TETRunAction::~TETRunAction()
{}

G4Run* TETRunAction::GenerateRun()
{
	// generate run
	fRun = new TETRun();
	return fRun;
}


void TETRunAction::BeginOfRunAction(const G4Run* aRun)
{
	// print the progress at the interval of 10%
	numOfEvent=aRun->GetNumberOfEventToBeProcessed();
	G4RunManager::GetRunManager()->SetPrintProgress(int(numOfEvent*0.1));
}

void TETRunAction::EndOfRunAction(const G4Run* aRun)
{
	// print the result only in the Master
	if(!isMaster) return;

	// get the run ID
	runID = aRun->GetRunID();

	// Print the run result by G4cout and std::ofstream
	//
	// print by G4cout
	PrintResult(G4cout);

	// print by std::ofstream
	std::ofstream ofs(outputFile.c_str());
	PrintResult(ofs);
	ofs.close();
}

void TETRunAction::PrintResult(std::ostream &out)
{
	// Print run result
	//
	using namespace std;
	EDEPMAP edepMap = *fRun->GetEdepMap();

	out << G4endl
	    << "=====================================================================" << G4endl
	    << " Run #" << runID << " / Number of event processed: "<< numOfEvent     << G4endl
	    << "=====================================================================" << G4endl
		<< " Organ dose - energy deposition to spongiosa/medullary cavity"         << G4endl
	    << "=====================================================================" << G4endl
	    << " organ ID| "
		<< setw(19) << "Organ Mass (g)"
		<< setw(19) << "Dose (Gy/source)"
		<< setw(19) << "Relative Error" << G4endl;

	out.precision(3);
	std::vector<int> DRFID = tetData->GetDRFID();
	sort(DRFID.begin(), DRFID.end());
	auto massMap = tetData->GetMassMap();
	for(auto itr : DRFID){

		G4String Name = to_string(itr);

		G4double meanDose    = edepMap[itr].first  / massMap[itr] / numOfEvent;
		G4double squareDoese = edepMap[itr].second / (massMap[itr]*massMap[itr]);
		G4double variance    = ((squareDoese/numOfEvent) - (meanDose*meanDose))/numOfEvent;
		G4double relativeE   = sqrt(variance)/meanDose;
		out << setw(9)  << Name << "| "
		<< setw(19) << fixed      << massMap[itr]/g;
		out	<< setw(19) << scientific << meanDose/(joule/kg);
		out	<< setw(19) << fixed      << relativeE << G4endl;
	}

	out<<G4endl
		<< "=====================================================================" << G4endl
		<< " Organ dose - fluence-to-dose response function: red bone marrow"         << G4endl
	    << "=====================================================================" << G4endl
		<< " organ ID| "
		<< setw(19) << "Organ Mass (g)"
		<< setw(19) << "Dose (Gy/source)"
		<< setw(19) << "Relative Error" << G4endl;
	for(auto itr:DRFID){
		G4String Name = to_string(itr);

		G4double meanDose    = edepMap[itr+1].first  / numOfEvent;
		G4double squareDoese = edepMap[itr+1].second;
		G4double variance    = ((squareDoese/numOfEvent) - (meanDose*meanDose))/numOfEvent;
		G4double relativeE   = sqrt(variance)/meanDose;
		out << setw(9)  << Name << "| "
			<< setw(19) << fixed      << massMap[itr]/g;
		out	<< setw(19) << scientific << meanDose/(joule/kg);
		out	<< setw(19) << fixed      << relativeE << G4endl;
	}

	out<<G4endl
		<< "=====================================================================" << G4endl
		<< " Organ dose - fluence-to-dose response function: bone endosteum"         << G4endl
	    << "=====================================================================" << G4endl
		<< " organ ID| "
		<< setw(19) << "Organ Mass (g)"
		<< setw(19) << "Dose (Gy/source)"
		<< setw(19) << "Relative Error" << G4endl;
	for(auto itr:DRFID){
		G4String Name = to_string(itr);

		G4double meanDose    = edepMap[itr+2].first  / numOfEvent;
		G4double squareDoese = edepMap[itr+2].second;
		G4double variance    = ((squareDoese/numOfEvent) - (meanDose*meanDose))/numOfEvent;
		G4double relativeE   = sqrt(variance)/meanDose;
		out << setw(9)  << Name << "| "
			<< setw(19) << fixed      << massMap[itr]/g;
		out	<< setw(19) << scientific << meanDose/(joule/kg);
		out	<< setw(19) << fixed      << relativeE << G4endl;
	}

	out << "=====================================================================" << G4endl << G4endl;
}
